#include <ArduinoBLE.h>

BLEService bleServiceTest("776982a9-8315-4bad-80f7-325e6a521aa6");
BLECharacteristic bleCharacteristicSwitch("df172997-36a6-446c-b70c-ac1556012427", BLERead | BLEWrite, 32);

bool bDisconnected = false;

void bleOnConnected(BLEDevice bleDevice)
{
  Serial.println("bleOnConnected: bleDevice.address()=" + bleDevice.address());
}

void bleOnDisconnected(BLEDevice bleDevice)
{
  Serial.println("bleOnDisconnected: bleDevice.address()=" + bleDevice.address());
  BLE.end();
  bDisconnected = true;
}

void bleOnCharacteristicStringWritten(BLEDevice bleDevice, BLECharacteristic characteristic)
{
  String sWritten = String (((char*)characteristic.value()), characteristic.valueLength());
  Serial.println("bleOnCharacteristicStringWritten: " + sWritten);
}

void bleOnCharacteristicStringRead(BLEDevice bleDevice, BLECharacteristic characteristic)
{
  characteristic.setValue(String("<123454321>").c_str());
  String sRead = String (((char*)characteristic.value()), characteristic.valueLength());
  Serial.println("bleOnCharacteristicStringRead: " + sRead);
}

void beginBLE()
{
  Serial.println("setup: calling BLE.begin()");
  if (!BLE.begin())
    Serial.println("BLE.begin() failed");
  BLE.setLocalName("BLE Server Test");
  BLE.setAdvertisedService(bleServiceTest);
  bleServiceTest.addCharacteristic(bleCharacteristicSwitch);
  BLE.addService(bleServiceTest);
  BLE.setEventHandler(BLEConnected, bleOnConnected);
  BLE.setEventHandler(BLEDisconnected, bleOnDisconnected);
  
  bleCharacteristicSwitch.setEventHandler(BLEWritten, bleOnCharacteristicStringWritten);
  bleCharacteristicSwitch.setEventHandler(BLERead, bleOnCharacteristicStringRead);
  bleCharacteristicSwitch.setValue(String("INIT").c_str());
  
  BLE.advertise();
}

void setup() 
{
  Serial.begin(115200);
  while (!Serial) 
    delay(10);

  beginBLE();
}

void loop() 
{
  static uint32_t uLoopCount = 0;
  uLoopCount++;
  
  if (uLoopCount == 1)
    Serial.println("loop: first call"); 

  if (bDisconnected == true)
  {
    bDisconnected = false;
    Serial.println("BLE.begin() in 5s");
    delay(5000);
    Serial.println("BLE.begin()");
    beginBLE();
  }
    
  BLE.poll();

  delay(10);
}